/* eslint-disable no-console */
import {
  Http, IDictionary, IEventParam, Metadata,
  DatasourceFactory, Router,
} from '@zeedhi/core';
import {
  Select, Grid, Container, Button, AlertService, Form,
  TextInput, Date as DateInput, Switch, SelectMultiple,
  StateType, ApexChart, DialogService, Card,
} from '@zeedhi/common';

import { getRouteParams } from '../router';

export class Grafico {
  public trueFalseValue: StateType = 'Não';

  public dataSelects: IDictionary = {
    selectUnidade: [],
    selectAtividades: [],
    selectLocal: [],
    selectUsuariosPermitidos: [],
  };

  public dataFilter: IDictionary = {
    dado: '',
  };

  public itensDelete: String = '';

  public consulta: String = '';

  public metricas: IDictionary = [];

  public metricasUpdate: IDictionary = [];

  private statesDatasource = DatasourceFactory.factory({
    uniqueKey: 'id',
    data: [
      {
        region_id: 4,
        id: 1,
        name: 'São Paulo',
        population: 3929911,
      },
      {
        region_id: 4,
        id: 2,
        name: 'Minas Gerais',
        population: 16550024,
      },
      {
        region_id: 4,
        id: 3,
        name: 'Rio de Janeiro',
        population: 20869101,
      },
      {
        region_id: 4,
        id: 15,
        name: 'Espírito Santo',
        population: 44396484,
      },
    ],
  });

  private dataGrafico: IDictionary = {
    data: [],
  };

  public initializeData() {
    this.getDadosUnidade();
    this.getDadosAtividades();
    this.getDadosLocal();
  }

  public setDataSelects({ component }: IEventParam<Select>) {
    component.datasource.data = this.dataSelects[component.name];
  }

  public setData(nm: string, id: number) {
    return {
      name: nm,
      value: id,
    };
  }

  // Método responsavel por pegar as informações de unidade e inserir ao select
  public getDadosUnidade() {
    Http.get('/getUnidade').then((response: IDictionary) => {
      response.data.dataset.dados.forEach((dados: IDictionary) => {
        this.dataSelects.selectUnidade.push(this.setData(dados.NMLOCALCLIENTE, dados.NRLOCALCLIENTE));
      });
    });
  }

  // Método responsavel por pegar as informações de atividades e inserir ao select
  public getDadosAtividades() {
    Http.get('/getAtividades').then((response: IDictionary) => {
      response.data.dataset.dados.forEach((dados: IDictionary) => {
        this.dataSelects.selectAtividades.push(this.setData(dados.NMATIVIDADE, dados.NRATIVIDADE));
      });
    });
  }

  // Método responsavel por pegar as informações de local e inserir ao select
  public getDadosLocal() {
    Http.get('/getLocal').then((response: IDictionary) => {
      response.data.dataset.local.forEach((dados: IDictionary) => {
        this.dataSelects.selectLocal.push(this.setData(dados.NMLOCALCLIENTE, dados.NRLOCALCLIENTE));
      });
    });
  }

  // Método responsavel por pegar as informações dos grafisos gerados e inserir ao select
  public getDadosGrid() {
    const informacaoBusca = this.dataFilter.dado;
    const selectObjectGrid = Metadata.getInstance<Grid>('grid');
    selectObjectGrid.reload();
    Http.post('/getDadosGraficosGerados', { informacaoBusca }).then((response: IDictionary) => {
      const { dados } = response.data.dataset;
      if (dados.length) {
        dados.forEach((data: IDictionary) => {
          selectObjectGrid.datasource.data.push({
            NRGRAFICOS: data.NRGRAFICOS,
            nome: data.NMGRAFICOS,
            tipoGrafico: data.TIPO,
            consulta: data.TXCONSULTAGRAFICO,
          });
        });
      } else {
        selectObjectGrid.datasource.data = [];
      }
    });
  }

  // Método responsavel por pegar as informações de usuários e inserir ao select
  public getDadosPermissaoUsuario() {
    const selectObjectUsuarios = Metadata.getInstance<SelectMultiple>('selectUsuariosPermitidos');
    Http.get('/usersList').then((response: IDictionary) => {
      response.data.dataset.info.forEach((usuario: IDictionary) => {
        selectObjectUsuarios.datasource.data.push(this.setData(usuario.NMFUNCIONARIO, usuario.NRFUNCIONARIO));
      });
    });
  }

  // Método responsavel por pegar as informações da fonte de dados e inserir ao select
  public getFonteDados() {
    const selectObjectFonte = Metadata.getInstance<Select>('selectFonte');
    Http.get('/getFonteDados').then((response: IDictionary) => {
      const { dados } = response.data.dataset;
      if (dados.length) {
        dados.forEach((fonte: IDictionary) => {
          selectObjectFonte.datasource.data.push(this.setData(fonte.NMFONTEDADOSGRAFREL, fonte.NRFONTEDADOSGRAFREL));
        });
      } else {
        selectObjectFonte.datasource.data = [];
      }
    });
  }

  // Método responsavel por deletar o grafico gerado
  public deleteGraficoGerado() {
    const gridObject = Metadata.getInstance<Grid>('grid');
    const NRGRAFICOS = this.itensDelete;
    Http.post('/deletaGraficoGerado', { NRGRAFICOS }).then((response:IDictionary) => {
      const { retorno } = response.data.dataset.resposta;
      if (retorno === 'Deletado') {
        const texto = 'Gráficos excluídos com sucesso!';
        const sucesso = 'success';
        this.mensagemAlert(texto, sucesso);
        gridObject.reload();
        this.getDadosGrid();
      } else {
        const texto = 'Ocorreu um erro ao excluir os itens';
        const sucesso = 'error';
        this.mensagemAlert(texto, sucesso);
      }
    });
  }

  // Função responsável por realizar a remoção e edição do grafico quando clicado no grid
  public async mostraGrafico() {
    const gridObject = Metadata.getInstance<Grid>('grid');
    const sectorInfos = gridObject.datasource.currentRow;
    Promise.all([this.getDadosExibicaoGrafico(sectorInfos.consulta)]).then(() => {
      const areaGrafico = Metadata.getInstance<Container>('ContainerInterno');
      if (areaGrafico.children.length === 0) {
        this.verificaGrafico();
      } else {
        this.removeFilho();
        this.verificaGrafico();
      }
    });
  }

  // modelo a ser inserid quando escolhemos um grafico tipo barra
  public modeloGraficoPizza(tipo: string) {
    return {
      name: 'modeloPizza',
      component: 'ZdApexChart',
      chartType: `${tipo}`,
      grid: {
        sm: '12',
        lg: '6',
      },
      options: {
        chart: {
        },
        labels: [
        ],
        responsive: [
          {
            breakpoint: '480',
            options: {
              chart: {
                width: '200',
              },
              legend: {
                position: 'bottom',
              },
            },
          },
        ],
      },
      series: [
      ],
    };
  }

  // modelo a ser inserid quando escolhemos um grafico tipo barra
  public modeloGraficoBarra(tipo: string) {
    return {
      name: 'modeloBarra',
      component: 'ZdApexChart',
      chartType: `${tipo}`,
      grid: {
        sm: '12',
        lg: '6',
      },
      options: {
        chart: {
          toolbar: {
            show: 'false',
          },
        },
        plotOptions: {
          bar: {
            columnWidth: '55%',
            endingShape: 'rounded',
          },
        },
      },
      xaxis: {
        categories: [
        ],
      },
      series: [
      ],
    };
  }

  // Verifica o tipo de grafico e insere as informações passadas previamente a um datasource
  public async verificaGrafico() {
    const gridObject = Metadata.getInstance<Grid>('grid');
    const sectorInfos = gridObject.datasource.currentRow;
    const areaGrafico = Metadata.getInstance<Container>('ContainerInterno');
    if (sectorInfos.tipoGrafico === 'Linha') {
      await areaGrafico.addChild(this.modeloGraficoBarra('line'));
      const selectObjectTipoGrafico = Metadata.getInstance<any>('modeloBarra');
      if (areaGrafico.children.length !== 0) {
        const stateLabels = this.dataGrafico.data.map((state: any) => state.nome);
        const stateSeries = this.dataGrafico.data.map((state: any) => state.data);
        await selectObjectTipoGrafico.updateChart({
          xaxis: {
            categories: stateLabels,
          },
          series: [{
            name: 'SERIE-1',
            data: stateSeries,
          }],
        });
      }
    } else if (sectorInfos.tipoGrafico === 'Barra') {
      await areaGrafico.addChild(this.modeloGraficoBarra('bar'));
      if (areaGrafico.children.length !== 0) {
        const selectObjectTipoGrafico = Metadata.getInstance<any>('modeloBarra');
        const stateLabels = this.dataGrafico.data.map((state: any) => state.nome);
        const stateSeries = this.dataGrafico.data.map((state: any) => state.data);
        await selectObjectTipoGrafico.updateChart({
          xaxis: {
            categories: stateLabels,
          },
          series: [{
            name: 'SERIE-1',
            data: stateSeries,
          }],
        });
      }
    } else if (sectorInfos.tipoGrafico === 'Area') {
      await areaGrafico.addChild(this.modeloGraficoBarra('area'));
      const selectObjectTipoGrafico = Metadata.getInstance<any>('modeloBarra');
      if (areaGrafico.children.length !== 0) {
        const stateLabels = this.dataGrafico.data.map((state: any) => state.nome);
        const stateSeries = this.dataGrafico.data.map((state: any) => state.data);
        await selectObjectTipoGrafico.updateChart({
          xaxis: {
            categories: stateLabels,
          },
          series: [{
            name: 'SERIE-1',
            data: stateSeries,
          }],
        });
      }
    } else if (sectorInfos.tipoGrafico === 'Pizza') {
      await areaGrafico.addChild(this.modeloGraficoPizza('pie'));
      const selectObjectTipoGrafico = Metadata.getInstance<any>('modeloPizza');
      const stateLabels = this.dataGrafico.data.map((state: any) => state.nome);
      const stateSeries = this.dataGrafico.data.map((state: any) => state.data);
      await selectObjectTipoGrafico.updateChart({
        labels: stateLabels,
        series: stateSeries,
      });
    } else if (sectorInfos.tipoGrafico === 'Rosquinha') {
      await areaGrafico.addChild(this.modeloGraficoPizza('donut'));
      const selectObjectTipoGrafico = Metadata.getInstance<any>('modeloPizza');
      const stateLabels = this.dataGrafico.data.map((state: any) => state.nome);
      const stateSeries = this.dataGrafico.data.map((state: any) => state.data);
      await selectObjectTipoGrafico.updateChart({
        labels: stateLabels,
        series: stateSeries,
      });
    } else if (sectorInfos.tipoGrafico === 'Radial') {
      await areaGrafico.addChild(this.modeloGraficoPizza('radialBar'));
      const selectObjectTipoGrafico = Metadata.getInstance<any>('modeloPizza');
      const stateLabels = this.dataGrafico.data.map((state: any) => state.nome);
      const stateSeries = this.dataGrafico.data.map((state: any) => state.data);
      await selectObjectTipoGrafico.updateChart({
        labels: stateLabels,
        series: stateSeries,
      });
    }
  }

  // Método responsável por remover o filho adicionado sempre quando um tipo de gráfico diferente é adicionado
  public removeFilho() {
    const areaGrafico = Metadata.getInstance<Container>('ContainerInterno');
    areaGrafico.children.pop();
  }

  // Método de mensagem simples
  public mensagemAlert(text: string, tipo: string) {
    AlertService.show({
      name: 'basic-alert',
      text: `${text}`,
      top: true,
      left: false,
      timeout: 3000,
      color: `${tipo}`,
    });
  }

  // Função não sendo utilizada no momento;
  public abreNovasOpcoesGraficos(tipo: string) {
    const tipoGrafico = tipo;
    const selectObjectSubTipoGrafico = Metadata.getInstance<Select>('selectsubTipoGrafico');
    selectObjectSubTipoGrafico.datasource.data = [];
    if (tipoGrafico === '1' || tipoGrafico === '2' || tipoGrafico === '3') {
      selectObjectSubTipoGrafico.isVisible = true;
      selectObjectSubTipoGrafico.value = '';
      selectObjectSubTipoGrafico.datasource.data.push({
        name: 'Grafico de Linha/Coluna',
        value: 1,
      },
      {
        name: 'Grafico de Linha/Coluna/Area',
        value: 2,
      });
      const data: any = [];
      this.statesDatasource.addFilter('region_id', 4).then(() => {
        const stateLabels = this.statesDatasource.data.map((state: any) => state.name);
        const stateSeries = this.statesDatasource.data.map((state: any) => state.population);
        data.push({
          name: 'valor',
          data: stateSeries,
        });
        const chart = Metadata.getInstance('states-chart') as ApexChart;
        chart.chartType = 'line';
        chart.updateChart({
          series: data,
          xaxis: {
            categories: stateLabels,
          },
        });
      });
    } else if (tipoGrafico === '5') {
      selectObjectSubTipoGrafico.isVisible = true;
      selectObjectSubTipoGrafico.value = '';
      selectObjectSubTipoGrafico.datasource.data.push({
        name: 'Grafico de Rosca',
        value: 1,
      },
      {
        name: 'Grafico Monochrome',
        value: 2,
      });
    } else if (tipoGrafico === '6') {
      selectObjectSubTipoGrafico.isVisible = true;
      selectObjectSubTipoGrafico.value = '';
      selectObjectSubTipoGrafico.datasource.data.push({
        name: 'Radar Multiplos',
        value: 1,
      },
      {
        name: 'Gradiente',
        value: 2,
      },
      {
        name: 'Medida',
        value: 3,
      });
    }
  }

  // Método reponsável por realizar a troca do valor do texto de grafico público
  public get trueFalseValueText() {
    return `<strong>${this.trueFalseValue}</strong>`;
  }

  private regionSelected = true;

  // Método responsavel por renderizar o grafico de acordo com o tipo selecionado
  public montaGraficoModelo() {
    const selectObjectAreaGrafico = Metadata.getInstance<Card>('cardGrafico');
    selectObjectAreaGrafico.removeChild('modelo');
    //  selectObjectMetricaY = Metadata.getInstance<Select>('metricaY');
    // const selectObjectCol = Metadata.getInstance<Col>('colMetricas');
    // selectObjectCol.cols = 6;
    // selectObjectMetricaY.isVisible = true;
    const selectObjectTipoGrafico = Metadata.getInstance<Select>('selectTipoChart').value;
    if (selectObjectTipoGrafico === '1') {
      selectObjectAreaGrafico.addChild({
        name: 'modelo',
        component: 'ZdApexChart',
        chartType: 'line',
        grid: {
          sm: '12',
        },
        options: {
          chart: {
            toolbar: {
              show: 'false',
            },
          },
          xaxis: {
            categories: [
              '1991',
              '1992',
              '1993',
              '1994',
              '1995',
              '1996',
              '1997',
              '1998',
            ],
          },
        },
        series: [
          {
            name: 'series-1',
            data: [
              '30',
              '40',
              '45',
              '50',
              '49',
              '60',
              '70',
              '91',
            ],
          },
        ],
      });
    } else if (selectObjectTipoGrafico === '2') {
      selectObjectAreaGrafico.addChild({
        name: 'modelo',
        component: 'ZdApexChart',
        chartType: 'bar',
        grid: {
          sm: '12',
        },
        options: {
          chart: {
            toolbar: {
              show: 'false',
            },
          },
          xaxis: {
            categories: [
              '1991',
              '1992',
              '1993',
              '1994',
              '1995',
              '1996',
              '1997',
              '1998',
            ],
          },
        },
        series: [
          {
            name: 'series-1',
            data: [
              '30',
              '40',
              '45',
              '50',
              '49',
              '60',
              '70',
              '91',
            ],
          },
        ],
      });
    } else if (selectObjectTipoGrafico === '3') {
      selectObjectAreaGrafico.children.push({
        name: 'modelo',
        component: 'ZdApexChart',
        chartType: 'area',
        grid: {
          sm: '12',
        },
        options: {
          chart: {
            toolbar: {
              show: 'false',
            },
          },
          xaxis: {
            categories: [
              '1991',
              '1992',
              '1993',
              '1994',
              '1995',
              '1996',
              '1997',
              '1998',
            ],
          },
        },
        series: [
          {
            name: 'series-1',
            data: [
              '30',
              '40',
              '45',
              '50',
              '49',
              '60',
              '70',
              '91',
            ],
          },
        ],
      });
    } else if (selectObjectTipoGrafico === '4') {
      // selectObjectMetricaY.isVisible = false;
      // selectObjectMetricaY.clearValidations();
      // selectObjectCol.cols = 12;
      selectObjectAreaGrafico.children.push({
        name: 'modelo',
        component: 'ZdApexChart',
        chartType: 'pie',
        grid: {
          sm: '12',
        },
        options: {
          labels: [
            'Team A',
            'Team B',
            'Team C',
            'Team D',
            'Team E',
          ],
        },
        series: [
          '44',
          '55',
          '13',
          '43',
          '22',
        ],
      });
    } else if (selectObjectTipoGrafico === '5') {
      // selectObjectMetricaY.isVisible = false;
      // selectObjectMetricaY.clearValidations();
      // selectObjectCol.cols = 12;
      selectObjectAreaGrafico.addChild({
        name: 'modelo',
        component: 'ZdApexChart',
        chartType: 'radialBar',
        grid: {
          sm: '12',
        },
        options: {
          chart: {
            id: 'radial-bar-chart',
          },
          plotOptions: {
            radialBar: {
              hollow: {
                size: '70%',
              },
            },
          },
          labels: [
            'Cricket',
          ],
        },
        series: [
          '70',
        ],
      });
    }
  }

  // Método responsável por habilitar e desabilitar itens de (Copiar/Editar/Deletar) do grid de graficos gerados
  public selecionaItemGrid({ component }: any) {
    this.itensDelete = [] as never;
    const selectObjectButtonDelete = Metadata.getInstance<Button>('sectorDelete');
    const selectObjectButtonCopy = Metadata.getInstance<Button>('sectorCopy');
    const selectObjectButtonEdit = Metadata.getInstance<Button>('sectorEdit');
    if (component.selectedRows.length !== 0) {
      selectObjectButtonDelete.disabled = false;
      selectObjectButtonCopy.disabled = false;
      selectObjectButtonEdit.disabled = false;
      const linhasSelecionadas = component.selectedRows.map((item: any) => item.NRGRAFICOS).join(', ');
      this.itensDelete = linhasSelecionadas;
    } else if (component.selectedRows.length === 0) {
      selectObjectButtonDelete.disabled = true;
      selectObjectButtonCopy.disabled = true;
      selectObjectButtonEdit.disabled = true;
    }
  }

  public mensagem(texto: string) {
    // const resposta = 'SIM';
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      text: `${texto}`,
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => this.deleteGraficoGerado(),
          },
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: {
            click: () => DialogService.hide(),
          },
        },
      ],
    });
  }

  // Método para limpar os campos após a inserção do grafico
  public limpaCampos() {
    Metadata.getInstance<TextInput>('nomeGrafico').value = '';
    Metadata.getInstance<Select>('selectFonte').value = '';
    Metadata.getInstance<Select>('selectTipoChart').value = '';
    Metadata.getInstance<Select>('metricaX').value = '';
    Metadata.getInstance<Select>('metricaY').value = '';
    Metadata.getInstance<Select>('periodo').value = '';
    Metadata.getInstance<DateInput>('dateDe').value = '';
    Metadata.getInstance<DateInput>('dateAte').value = '';
    Metadata.getInstance<Switch>('trueFalseSwitch').value = 'Não';
    Metadata.getInstance<SelectMultiple>('selectUsuariosPermitidos').value = [''];
    Metadata.getInstance<ApexChart>('states-chart').isVisible = false;
  }

  // Método responsável por realizar verificação do switch de (grafico publico)
  public verificaSwitch({ component }:{ component:Switch }) {
    const selectObjectUsuarios = Metadata.getInstance<SelectMultiple>('selectUsuariosPermitidos');
    if (component.value === 'Sim') {
      selectObjectUsuarios.isVisible = false;
      selectObjectUsuarios.clearValidations();
    } else {
      selectObjectUsuarios.isVisible = true;
      selectObjectUsuarios.validate();
    }
  }

  public dadosSelect: IDictionary = {
    SELECT: [],
    FROM: [],
    JOIN: [],
    ON: [],
    GROUP: [],
    WHERE: [],
    AND: [],
  };

  public dadosConsulta : IDictionary = [];

  // Método responsavel por ler a tabela FCM_DADOSGRAFREL e retornas as informaçõe para montar a consulta
  public async buscaParametrosSQL(parametro: string) {
    await Http.post('/buscaDadosGrafRel', { parametro }).then((response: IDictionary) => {
      const dados = response.data.dataset.resposta;
      this.dadosConsulta = dados;
    });
  }

  // Método responsavel por verificar se o usuário não selecionou mais de um grafico para editar
  public editGrafico() {
    const gridObject = Metadata.getInstance<Grid>('grid');
    if (gridObject.selectedRows.length > 1) {
      const texto = 'Não é possivel editar mais de um grafico por vez!';
      this.mensagemAlert(texto, 'warning');
    } else {
      const NRGRAFICOS = this.itensDelete;
      Router.replace(`/editGrafico/${NRGRAFICOS}`);
    }
  }

  // Método para inserir os dados do grafico já inserido
  public async insereDadosEditGrafico() {
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    const params = {
      NRGRAFICOS: nr,
    };

    await Http.post('/getDadosEditGrafico', { params }).then((retorno: IDictionary) => {
      const dados = retorno.data.dataset.resposta;
      dados[0].forEach((dadosEdit: IDictionary) => {
        Metadata.getInstance<TextInput>('nomeGrafico').value = dadosEdit.NMGRAFICOS;
        Metadata.getInstance<Select>('selectFonte').setValue(dadosEdit.NRFONTEDADOSGRAFREL);
        const tipoGrafico = dadosEdit.IDTPGRAFICO;
        Metadata.getInstance<Select>('selectTipoChart').setValue(tipoGrafico.trim());
        Metadata.getInstance<Select>('periodo').setValue(dadosEdit.IDPERIODO);
        Metadata.getInstance<DateInput>('dateDe').value = dadosEdit.DATAINICIO;
        Metadata.getInstance<DateInput>('dateAte').value = dadosEdit.DATAFINAL;
        if (dadosEdit.IDGRAFICOPUB === 'S') {
          Metadata.getInstance<Switch>('trueFalseSwitch').value = 'Sim';
          const campofuncionario = Metadata.getInstance<SelectMultiple>('selectUsuariosPermitidos');
          campofuncionario.isVisible = false;
          campofuncionario.clearValidations();
        } else {
          Metadata.getInstance<Switch>('trueFalseSwitch').value = 'Não';
        }
      });
      dados[1].forEach((dadosMetrica: IDictionary) => {
        const dimensao = dadosMetrica.IDDIMENSAODADO;
        if (dimensao.trim() === 'X') {
          Metadata.getInstance<Select>('metricaX').setValue(dadosMetrica.NRDADOSGRAFREL);
          this.metricasUpdate.push({
            IDDIMENSAODADO: 'X',
            NRDADOSGRAFREL: dadosMetrica.NRDADOSGRAFREL,
          });
        } else if (dimensao.trim() === 'Y') {
          Metadata.getInstance<Select>('metricaY').setValue(dadosMetrica.NRDADOSGRAFREL);
          this.metricasUpdate.push({
            IDDIMENSAODADO: 'Y',
            NRDADOSGRAFREL: dadosMetrica.NRDADOSGRAFREL,
          });
        }
      });
      if (dados[2] !== []) {
        const func: IDictionary = [];
        dados[2].forEach((funcionarios: IDictionary) => {
          func.push(funcionarios.NRFUNCIONARIO);
        });
        Metadata.getInstance<SelectMultiple>('selectUsuariosPermitidos').setValue(func);
      }
    });
    console.log(this.metricasUpdate);
  }

  public atualizaGrafico() {
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    Promise.all([this.montaConsulta()]).then(() => {
      const selectObjectForm = Metadata.getInstance<Form>('form');
      if (selectObjectForm.validate()) {
        Http.post('/atualizaGrafico', {
          NRGRAFICOS: nr,
          NMGRAFICOS: Metadata.getInstance<TextInput>('nomeGrafico').value,
          NRFONTEDADOSGRAFREL: Metadata.getInstance<Select>('selectFonte').value,
          IDTPGRAFICO: Metadata.getInstance<Select>('selectTipoChart').value,
          IDPERIODO: Metadata.getInstance<Select>('periodo').value,
          NRORG: 20,
          IDATIVO: 'S',
          CDOPERINCLUSAO: '000000004280',
          NRORGINCLUSAO: 20,
          METRICAS: this.metricasUpdate,
          DTINIGRAFICO: Metadata.getInstance<DateInput>('dateDe').value,
          DTFIMGRAFICO: Metadata.getInstance<DateInput>('dateAte').value,
          IDGRAFICOPUB: Metadata.getInstance<Switch>('trueFalseSwitch').value.slice(0, 1),
          NRFUNCIONARIO: Metadata.getInstance<SelectMultiple>('selectUsuariosPermitidos').value,
          TXCONSULTAGRAFICO: this.consulta,
        }).then((response: IDictionary) => {
          const { retorno } = response.data.dataset.resposta;
          if (retorno === 'Atualizado') {
            const texto = 'Grafico Atualizado com sucesso!';
            const tipo = 'success';
            this.mensagemAlert(texto, tipo);
          } else {
            const texto = 'Ocorreu um erro ao atualizar o grafico';
            const tipo = 'error';
            this.mensagemAlert(texto, tipo);
          }
        });
      } else {
        const texto = 'Preencha todos os campos';
        const tipo = 'error';
        this.mensagemAlert(texto, tipo);
      }
    });
  }

  // Método para realizar a cópia do grafico
  public copiaGrafico() {
    const gridObject = Metadata.getInstance<Grid>('grid');
    const NRGRAFICOS = this.itensDelete;
    Http.post('/copiaGrafico', { NRGRAFICOS }).then((resposta: any) => {
      if (resposta.data.dataset.resposta.retorno === 'Copiado') {
        const texto = 'Grafico copido com sucesso!';
        this.mensagemAlert(texto, 'success');
        gridObject.reload();
        this.getDadosGrid();
      } else {
        const texto = 'Não foi possivel cópiar esse grafico';
        this.mensagemAlert(texto, 'warning');
      }
    });
  }

  // responsavel por pegar os dados que são digitados e adicionado a um dicionario e chamar a função que realiza a busca
  public busca({ component }: any) {
    this.dataFilter.dado = '';
    this.dataFilter.dado = component.value;
    this.getDadosGrid();
  }

  public pegaMetricasGrafico({ component }: IEventParam<Select>) {
    if (component.name === 'metricaX') {
      this.metricas.push({
        IDDIMENSAODADO: 'X',
        NRDADOSGRAFREL: component.value,
      });
    } else if (component.name === 'metricaY') {
      this.metricas.push({
        IDDIMENSAODADO: 'Y',
        NRDADOSGRAFREL: component.value,
      });
    } else {
      this.metricas = [];
    }
  }

  // Método para montar a consulta do grafico que esta sendo gerado
  public async montaConsulta() {
    const selectObjectDataDe = Metadata.getInstance<DateInput>('dateDe').value;
    const padrao = selectObjectDataDe.split('/').reverse().join('/');
    const dataInicial = new Date(`${padrao}`).toDateString();
    const selectObjectDataAte = Metadata.getInstance<DateInput>('dateAte').value;
    const padrao1 = selectObjectDataAte.split('/').reverse().join('/');
    const dataFinal = new Date(`${padrao1}`).toDateString();
    const selectObjectMetricaX = Metadata.getInstance<Select>('metricaX').value;
    const selectObjectMetricaY = Metadata.getInstance<Select>('metricaY').value;
    const parametro = `${selectObjectMetricaX},${selectObjectMetricaY}`;
    await Promise.all([this.buscaParametrosSQL(parametro)]).then(() => {
      const dados = this.dadosConsulta;
      dados.forEach((parametros: IDictionary) => {
        if (parametros.DSSQLCUSTOMIZADO !== null) {
          if (this.dadosSelect.SELECT.length !== 0) {
            const valorSelect = this.dadosSelect.SELECT;
            this.dadosSelect.SELECT = [];
            this.dadosSelect.SELECT = valorSelect.concat(`, ${parametros.DSSQLCUSTOMIZADO}`);
          }
        } else if (parametros.DSSQLCUSTOMIZADO === null) {
          this.dadosSelect.FROM = `FROM ${parametros.NMTABELADADO}`;
          if (parametros.NMCAMPODADO !== null) {
            this.dadosSelect.SELECT = `${parametros.NMTABELANOMEDADO}.${parametros.NMCAMPONOMEDADO}`;
            if (parametros.NMTABELANOMEDADO !== null) {
              this.dadosSelect.JOIN = ` JOIN ${parametros.NMTABELANOMEDADO}`;
              this.dadosSelect.ON = ` ON ${parametros.NMTABELADADO}.${parametros.NMCAMPODADO} = ${parametros.NMTABELANOMEDADO}.${parametros.NMCAMPODADO}`;
              this.dadosSelect.GROUP = ` GROUP BY ${parametros.NMTABELANOMEDADO}.${parametros.NMCAMPONOMEDADO}`;
              if (parametros.NMTABELADATAFILTRO !== null) {
                if (dataInicial === dataFinal) {
                  this.dadosSelect.WHERE = ` WHERE TRUNC(${parametros.NMTABELADATAFILTRO}.${parametros.NMCAMPODATAFILTRO}) = TO_DATE('${selectObjectDataDe}','DD/MM/YY')`;
                } else {
                  this.dadosSelect.WHERE = `WHERE TRUNC(${parametros.NMTABELADATAFILTRO}.${parametros.NMCAMPODATAFILTRO}) BETWEEN TO_DATE('${selectObjectDataDe}','DD/MM/YY') AND TO_DATE('${selectObjectDataAte}','DD/MM/YY')`;
                }
              }
              // console.log(this.dadosSelect.JOIN, this.dadosSelect.ON, this.dadosSelect.GROUP, this.dadosSelect.WHERE);
            }
          }
        }
      });
    });
    this.consulta = `SELECT ${this.dadosSelect.SELECT} ${this.dadosSelect.FROM} ${this.dadosSelect.JOIN} ${this.dadosSelect.ON} ${this.dadosSelect.WHERE} ${this.dadosSelect.GROUP}`;
    console.log(this.consulta);
  }

  // Método responsavel por verificar o fluxo de preenchimento dos campos(FONTE/METRICAX/METRICAY)
  public verificaSelect({ component }: IEventParam<Select>) {
    const data = component.datasource.data.length;
    const selectObjectFonteDados = Metadata.getInstance<Select>('selectFonte').label;
    const selectObjectMetricaX = Metadata.getInstance<Select>('metricaX').label;
    if (data === 0 && component.name === 'metricaX') {
      const texto = `Favor preencher o campo <b>(${selectObjectFonteDados})<b>`;
      const tipo = 'info';
      this.mensagemAlert(texto, tipo);
    }
    if (data === 0 && component.name === 'metricaY') {
      const texto = `Favor preencher o campo <b>(${selectObjectMetricaX})<b>`;
      const tipo = 'info';
      this.mensagemAlert(texto, tipo);
    }
  }

  // Método responsavel por validar campos preenchidos do formulário e enviar as informações para o insert do grafico
  public insertNovoGrafico() {
    Promise.all([this.montaConsulta()]).then(() => {
      const selectObjectForm = Metadata.getInstance<Form>('form');
      if (selectObjectForm.validate()) {
        Http.post('/insertGrafico', {
          NMGRAFICOS: Metadata.getInstance<TextInput>('nomeGrafico').value,
          NRFONTEDADOSGRAFREL: Metadata.getInstance<Select>('selectFonte').value,
          IDTPGRAFICO: Metadata.getInstance<Select>('selectTipoChart').value,
          IDPERIODO: Metadata.getInstance<Select>('periodo').value,
          NRORG: 20,
          IDATIVO: 'S',
          CDOPERINCLUSAO: '000000004280',
          NRORGINCLUSAO: 20,
          METRICAS: this.metricas,
          DTINIGRAFICO: Metadata.getInstance<DateInput>('dateDe').value,
          DTFIMGRAFICO: Metadata.getInstance<DateInput>('dateAte').value,
          IDGRAFICOPUB: Metadata.getInstance<Switch>('trueFalseSwitch').value.slice(0, 1),
          NRFUNCIONARIO: Metadata.getInstance<SelectMultiple>('selectUsuariosPermitidos').value,
          TXCONSULTAGRAFICO: this.consulta,
        }).then((response: IDictionary) => {
          const { retorno } = response.data.dataset.resposta;
          if (retorno === 'OK') {
            const texto = 'Grafico cadastrado com sucesso!';
            const tipo = 'success';
            this.mensagemAlert(texto, tipo);
            this.limpaCampos();
          } else {
            const texto = 'Ocorreu um erro ao gerar o grafico';
            const tipo = 'error';
            this.mensagemAlert(texto, tipo);
          }
        });
      } else {
        const texto = 'Preencha todos os campos';
        const tipo = 'error';
        this.mensagemAlert(texto, tipo);
      }
    });
  }

  // Método responsavel por pegar as informações de unidade e inserir ao select
  public async getDadosExibicaoGrafico(consulta: IDictionary) {
    await Http.post('/consultaDinamica', { consulta }).then((responseDadosCard: IDictionary) => {
      const dadosGrafico = responseDadosCard.data.dataset.resposta;
      this.dataGrafico.data = [];
      dadosGrafico.forEach((dataGrafico: IDictionary) => {
        this.dataGrafico.data.push({
          nome: dataGrafico.NMCHECKLIST,
          data: dataGrafico.CTCHECKLIST,
        });
      });
    });
  }

  // Método responsável por pegar o dado da mética X
  public getMetricaX({ component }: IEventParam<Select>) {
    const selectObjectFonte = Metadata.getInstance<Select>('selectFonte').value;
    const selectObjectMetricaX = Metadata.getInstance<Select>('metricaX');
    this.pegaMetricasGrafico({ component });
    Http.post('/getMetricas', { selectObjectFonte }).then((response: IDictionary) => {
      const { dados } = response.data.dataset;
      if (dados.length) {
        dados.forEach((metrica: IDictionary) => {
          selectObjectMetricaX.datasource.data.push(this.setData(metrica.NMDADOSGRAFREL, metrica.NRDADOSGRAFREL));
        });
      } else {
        selectObjectMetricaX.datasource.data = [];
      }
    });
  }

  // Método responsável por pegar o dado da mética X
  public getMetricaY({ component }: IEventParam<Select>) {
    this.pegaMetricasGrafico({ component });
    Metadata.getInstance<Select>('metricaY').value = '';
    const selectObjectFonte = Metadata.getInstance<Select>('selectFonte').value;
    const selectObjectMetricaX = Metadata.getInstance<Select>('metricaX').value;
    const selectObjectMetricaY = Metadata.getInstance<Select>('metricaY');
    Http.post('/getMetricas', { selectObjectFonte }).then((response: IDictionary) => {
      const { dados } = response.data.dataset;
      if (dados.length) {
        dados.forEach((metrica: IDictionary) => {
          if (metrica.NRDADOSGRAFREL !== selectObjectMetricaX) {
            selectObjectMetricaY.datasource.data.push(this.setData(metrica.NMDADOSGRAFREL, metrica.NRDADOSGRAFREL));
          }
        });
      } else {
        selectObjectMetricaY.datasource.data = [];
      }
    });
  }

  // Método responsável por pegar o dado das méticas quando atualizamos o grafico
  public pegaMetricasGraficoUpdate({ component }: IEventParam<Select>) {
    if (component.value.length !== 0) {
      this.metricasUpdate = [];
    } else if (component.value.length === 0) {
      if (component.name === 'metricaX') {
        this.metricasUpdate.push({
          IDDIMENSAODADO: 'X',
          NRDADOSGRAFREL: component.value,
        });
      } else if (component.name === 'metricaY') {
        this.metricasUpdate.push({
          IDDIMENSAODADO: 'Y',
          NRDADOSGRAFREL: component.value,
        });
      } else {
        this.metricasUpdate = [];
      }
    }
  }

  public refreshData({ component }: { component: Select }) {
    component.datasource.get();
  }
}
